# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: wrapper.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import get_namespace as get_services_namespace
from accelbyte_py_sdk.core import run_request
from accelbyte_py_sdk.core import run_request_async
from accelbyte_py_sdk.core import deprecated
from accelbyte_py_sdk.core import same_doc_as

from ..models import ApimodelBulkSubscribeRequest
from ..models import ApimodelGetNotificationSubscriberListResponse
from ..models import ApimodelGetNotificationSubscriberStatusResponse
from ..models import ApimodelSelfSubscribeNotificationRequest
from ..models import ApimodelSubscribeNotificationRequest
from ..models import ApimodelSubscribeNotificationResponse
from ..models import ResponseErrorResponse

from ..operations.notification_subscription import BulkSaveSubscriptionAppNotificationV2
from ..operations.notification_subscription import (
    DeleteSubscriptionAppNotificationByUserIDV2,
)
from ..operations.notification_subscription import DeleteSubscriptionAppNotificationV2
from ..operations.notification_subscription import GetNotificationSubscriberListV2
from ..operations.notification_subscription import GetSubscriptionV2Handler
from ..operations.notification_subscription import SubscribeAppNotificationV2
from ..operations.notification_subscription import SubscribeV2Handler
from ..operations.notification_subscription import UnsubscribeV2Handler


@same_doc_as(BulkSaveSubscriptionAppNotificationV2)
def bulk_save_subscription_app_notification_v2(
    app: str,
    body: ApimodelBulkSubscribeRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk update the subscription of the users (BulkSaveSubscriptionAppNotificationV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SUBSCRIPTION [UPDATE]`

    Update the subscription the user(s) is an app notification.

    Request body:
    - notificationType : type of the app notification to be subscribed - Required.
    - values:
    - "all"
    - "app-down"
    - "critical-vulnerability"
    - subscribers : user(s) notification subscription to be updated - Required.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions

        method: PUT

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelBulkSubscribeRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelGetNotificationSubscriberListResponse

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkSaveSubscriptionAppNotificationV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(BulkSaveSubscriptionAppNotificationV2)
async def bulk_save_subscription_app_notification_v2_async(
    app: str,
    body: ApimodelBulkSubscribeRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Bulk update the subscription of the users (BulkSaveSubscriptionAppNotificationV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SUBSCRIPTION [UPDATE]`

    Update the subscription the user(s) is an app notification.

    Request body:
    - notificationType : type of the app notification to be subscribed - Required.
    - values:
    - "all"
    - "app-down"
    - "critical-vulnerability"
    - subscribers : user(s) notification subscription to be updated - Required.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions

        method: PUT

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelBulkSubscribeRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelGetNotificationSubscriberListResponse

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = BulkSaveSubscriptionAppNotificationV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteSubscriptionAppNotificationByUserIDV2)
def delete_subscription_app_notification_by_user_idv2(
    app: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Remove other person subscription by user ID (DeleteSubscriptionAppNotificationByUserIDV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SUBSCRIPTION [DELETE]`

    Remove a user from the notification subscription by user ID.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions/users/{userId}

        method: DELETE

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteSubscriptionAppNotificationByUserIDV2.create(
        app=app,
        user_id=user_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteSubscriptionAppNotificationByUserIDV2)
async def delete_subscription_app_notification_by_user_idv2_async(
    app: str,
    user_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Remove other person subscription by user ID (DeleteSubscriptionAppNotificationByUserIDV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SUBSCRIPTION [DELETE]`

    Remove a user from the notification subscription by user ID.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions/users/{userId}

        method: DELETE

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        user_id: (userId) REQUIRED str in path

    Responses:
        200: OK -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteSubscriptionAppNotificationByUserIDV2.create(
        app=app,
        user_id=user_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(DeleteSubscriptionAppNotificationV2)
def delete_subscription_app_notification_v2(
    app: str,
    subscription_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Remove other person subscription (DeleteSubscriptionAppNotificationV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SUBSCRIPTION [DELETE]`

    Remove a user from the notification subscription.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions/{subscriptionId}

        method: DELETE

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        subscription_id: (subscriptionId) REQUIRED str in path

    Responses:
        200: OK -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteSubscriptionAppNotificationV2.create(
        app=app,
        subscription_id=subscription_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(DeleteSubscriptionAppNotificationV2)
async def delete_subscription_app_notification_v2_async(
    app: str,
    subscription_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Remove other person subscription (DeleteSubscriptionAppNotificationV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SUBSCRIPTION [DELETE]`

    Remove a user from the notification subscription.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions/{subscriptionId}

        method: DELETE

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        subscription_id: (subscriptionId) REQUIRED str in path

    Responses:
        200: OK -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteSubscriptionAppNotificationV2.create(
        app=app,
        subscription_id=subscription_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetNotificationSubscriberListV2)
def get_notification_subscriber_list_v2(
    app: str,
    notification_type: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a list of the app notification subscriber (GetNotificationSubscriberListV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SUBSCRIPTION [READ]`

    Request param:
    - notificationType : type of the subscribed app notification
    - values:
    - all
    - app-down
    - critical-vulnerability

    Get a list of the app notification subscriber

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions

        method: GET

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        notification_type: (notificationType) REQUIRED str in query

    Responses:
        200: OK - ApimodelGetNotificationSubscriberListResponse

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetNotificationSubscriberListV2.create(
        app=app,
        notification_type=notification_type,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetNotificationSubscriberListV2)
async def get_notification_subscriber_list_v2_async(
    app: str,
    notification_type: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a list of the app notification subscriber (GetNotificationSubscriberListV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SUBSCRIPTION [READ]`

    Request param:
    - notificationType : type of the subscribed app notification
    - values:
    - all
    - app-down
    - critical-vulnerability

    Get a list of the app notification subscriber

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions

        method: GET

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        notification_type: (notificationType) REQUIRED str in query

    Responses:
        200: OK - ApimodelGetNotificationSubscriberListResponse

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetNotificationSubscriberListV2.create(
        app=app,
        notification_type=notification_type,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetSubscriptionV2Handler)
def get_subscription_v2_handler(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Subscription status of a user (GetSubscriptionV2Handler)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SELF:SUBSCRIPTION [READ]`

    Get Subscription status of a user

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions/me

        method: GET

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelGetNotificationSubscriberStatusResponse

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetSubscriptionV2Handler.create(
        app=app,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetSubscriptionV2Handler)
async def get_subscription_v2_handler_async(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Subscription status of a user (GetSubscriptionV2Handler)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SELF:SUBSCRIPTION [READ]`

    Get Subscription status of a user

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions/me

        method: GET

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelGetNotificationSubscriberStatusResponse

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetSubscriptionV2Handler.create(
        app=app,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(SubscribeAppNotificationV2)
def subscribe_app_notification_v2(
    app: str,
    body: ApimodelSubscribeNotificationRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Subscribe the user(s) an app notification (SubscribeAppNotificationV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SUBSCRIPTION [CREATE]`

    Subscribe the user(s) an app notification.

    Request body:
    - notificationType : type of the app notification to be subscribed - Required.
    - values:
    -"all"
    - "app-down"
    - "critical-vulnerability"
    - subscribers : user(s) to subscribe the app notification - Required.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions

        method: POST

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelSubscribeNotificationRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelSubscribeNotificationResponse

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = SubscribeAppNotificationV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(SubscribeAppNotificationV2)
async def subscribe_app_notification_v2_async(
    app: str,
    body: ApimodelSubscribeNotificationRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Subscribe the user(s) an app notification (SubscribeAppNotificationV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SUBSCRIPTION [CREATE]`

    Subscribe the user(s) an app notification.

    Request body:
    - notificationType : type of the app notification to be subscribed - Required.
    - values:
    -"all"
    - "app-down"
    - "critical-vulnerability"
    - subscribers : user(s) to subscribe the app notification - Required.

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions

        method: POST

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelSubscribeNotificationRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelSubscribeNotificationResponse

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = SubscribeAppNotificationV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(SubscribeV2Handler)
def subscribe_v2_handler(
    app: str,
    body: ApimodelSelfSubscribeNotificationRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Subscribe to app down notification (SubscribeV2Handler)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SELF:SUBSCRIPTION [CREATE]`

    Subscribe to app down notification
    Request body:
    - notificationType : type of the app notification to be subscribed - Required.
    - values:
    - "all"
    - "app-down"
    - "critical-vulnerability"

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions/me

        method: POST

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelSelfSubscribeNotificationRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (successfully submitted request to subcribe from app down notification)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = SubscribeV2Handler.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(SubscribeV2Handler)
async def subscribe_v2_handler_async(
    app: str,
    body: ApimodelSelfSubscribeNotificationRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Subscribe to app down notification (SubscribeV2Handler)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SELF:SUBSCRIPTION [CREATE]`

    Subscribe to app down notification
    Request body:
    - notificationType : type of the app notification to be subscribed - Required.
    - values:
    - "all"
    - "app-down"
    - "critical-vulnerability"

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions/me

        method: POST

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelSelfSubscribeNotificationRequest in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (successfully submitted request to subcribe from app down notification)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = SubscribeV2Handler.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UnsubscribeV2Handler)
def unsubscribe_v2_handler(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Unsubscribe to app down notification (UnsubscribeV2Handler)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SELF:SUBSCRIPTION [DELETE]`

    Unsubscribe to app down notification

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions/me

        method: DELETE

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (successfully submitted request to unsubcribe from app down notification)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UnsubscribeV2Handler.create(
        app=app,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UnsubscribeV2Handler)
async def unsubscribe_v2_handler_async(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Unsubscribe to app down notification (UnsubscribeV2Handler)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP:ALERT:SELF:SUBSCRIPTION [DELETE]`

    Unsubscribe to app down notification

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/subscriptions/me

        method: DELETE

        tags: ["Notification Subscription"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (successfully submitted request to unsubcribe from app down notification)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UnsubscribeV2Handler.create(
        app=app,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class ApimodelReplicaResponse(Model):
    """Apimodel replica response (apimodel.ReplicaResponse)

    Properties:
        current_replica: (currentReplica) OPTIONAL int

        max_replica: (maxReplica) OPTIONAL int

        min_replica: (minReplica) OPTIONAL int

        replica_limit: (replicaLimit) OPTIONAL int
    """

    # region fields

    current_replica: int  # OPTIONAL
    max_replica: int  # OPTIONAL
    min_replica: int  # OPTIONAL
    replica_limit: int  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_current_replica(self, value: int) -> ApimodelReplicaResponse:
        self.current_replica = value
        return self

    def with_max_replica(self, value: int) -> ApimodelReplicaResponse:
        self.max_replica = value
        return self

    def with_min_replica(self, value: int) -> ApimodelReplicaResponse:
        self.min_replica = value
        return self

    def with_replica_limit(self, value: int) -> ApimodelReplicaResponse:
        self.replica_limit = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "current_replica"):
            result["currentReplica"] = int(self.current_replica)
        elif include_empty:
            result["currentReplica"] = 0
        if hasattr(self, "max_replica"):
            result["maxReplica"] = int(self.max_replica)
        elif include_empty:
            result["maxReplica"] = 0
        if hasattr(self, "min_replica"):
            result["minReplica"] = int(self.min_replica)
        elif include_empty:
            result["minReplica"] = 0
        if hasattr(self, "replica_limit"):
            result["replicaLimit"] = int(self.replica_limit)
        elif include_empty:
            result["replicaLimit"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        current_replica: Optional[int] = None,
        max_replica: Optional[int] = None,
        min_replica: Optional[int] = None,
        replica_limit: Optional[int] = None,
        **kwargs,
    ) -> ApimodelReplicaResponse:
        instance = cls()
        if current_replica is not None:
            instance.current_replica = current_replica
        if max_replica is not None:
            instance.max_replica = max_replica
        if min_replica is not None:
            instance.min_replica = min_replica
        if replica_limit is not None:
            instance.replica_limit = replica_limit
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelReplicaResponse:
        instance = cls()
        if not dict_:
            return instance
        if "currentReplica" in dict_ and dict_["currentReplica"] is not None:
            instance.current_replica = int(dict_["currentReplica"])
        elif include_empty:
            instance.current_replica = 0
        if "maxReplica" in dict_ and dict_["maxReplica"] is not None:
            instance.max_replica = int(dict_["maxReplica"])
        elif include_empty:
            instance.max_replica = 0
        if "minReplica" in dict_ and dict_["minReplica"] is not None:
            instance.min_replica = int(dict_["minReplica"])
        elif include_empty:
            instance.min_replica = 0
        if "replicaLimit" in dict_ and dict_["replicaLimit"] is not None:
            instance.replica_limit = int(dict_["replicaLimit"])
        elif include_empty:
            instance.replica_limit = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelReplicaResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelReplicaResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelReplicaResponse,
        List[ApimodelReplicaResponse],
        Dict[Any, ApimodelReplicaResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "currentReplica": "current_replica",
            "maxReplica": "max_replica",
            "minReplica": "min_replica",
            "replicaLimit": "replica_limit",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "currentReplica": False,
            "maxReplica": False,
            "minReplica": False,
            "replicaLimit": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class ApimodelGetAppListV2Request(Model):
    """Apimodel get app list V2 request (apimodel.GetAppListV2Request)

    Properties:
        app_names: (appNames) OPTIONAL List[str]

        app_statuses: (appStatuses) OPTIONAL List[str]

        fuzzy_app_name: (fuzzyAppName) OPTIONAL str

        scenario: (scenario) OPTIONAL str
    """

    # region fields

    app_names: List[str]  # OPTIONAL
    app_statuses: List[str]  # OPTIONAL
    fuzzy_app_name: str  # OPTIONAL
    scenario: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_app_names(self, value: List[str]) -> ApimodelGetAppListV2Request:
        self.app_names = value
        return self

    def with_app_statuses(self, value: List[str]) -> ApimodelGetAppListV2Request:
        self.app_statuses = value
        return self

    def with_fuzzy_app_name(self, value: str) -> ApimodelGetAppListV2Request:
        self.fuzzy_app_name = value
        return self

    def with_scenario(self, value: str) -> ApimodelGetAppListV2Request:
        self.scenario = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "app_names"):
            result["appNames"] = [str(i0) for i0 in self.app_names]
        elif include_empty:
            result["appNames"] = []
        if hasattr(self, "app_statuses"):
            result["appStatuses"] = [str(i0) for i0 in self.app_statuses]
        elif include_empty:
            result["appStatuses"] = []
        if hasattr(self, "fuzzy_app_name"):
            result["fuzzyAppName"] = str(self.fuzzy_app_name)
        elif include_empty:
            result["fuzzyAppName"] = ""
        if hasattr(self, "scenario"):
            result["scenario"] = str(self.scenario)
        elif include_empty:
            result["scenario"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        app_names: Optional[List[str]] = None,
        app_statuses: Optional[List[str]] = None,
        fuzzy_app_name: Optional[str] = None,
        scenario: Optional[str] = None,
        **kwargs,
    ) -> ApimodelGetAppListV2Request:
        instance = cls()
        if app_names is not None:
            instance.app_names = app_names
        if app_statuses is not None:
            instance.app_statuses = app_statuses
        if fuzzy_app_name is not None:
            instance.fuzzy_app_name = fuzzy_app_name
        if scenario is not None:
            instance.scenario = scenario
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelGetAppListV2Request:
        instance = cls()
        if not dict_:
            return instance
        if "appNames" in dict_ and dict_["appNames"] is not None:
            instance.app_names = [str(i0) for i0 in dict_["appNames"]]
        elif include_empty:
            instance.app_names = []
        if "appStatuses" in dict_ and dict_["appStatuses"] is not None:
            instance.app_statuses = [str(i0) for i0 in dict_["appStatuses"]]
        elif include_empty:
            instance.app_statuses = []
        if "fuzzyAppName" in dict_ and dict_["fuzzyAppName"] is not None:
            instance.fuzzy_app_name = str(dict_["fuzzyAppName"])
        elif include_empty:
            instance.fuzzy_app_name = ""
        if "scenario" in dict_ and dict_["scenario"] is not None:
            instance.scenario = str(dict_["scenario"])
        elif include_empty:
            instance.scenario = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelGetAppListV2Request]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelGetAppListV2Request]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelGetAppListV2Request,
        List[ApimodelGetAppListV2Request],
        Dict[Any, ApimodelGetAppListV2Request],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "appNames": "app_names",
            "appStatuses": "app_statuses",
            "fuzzyAppName": "fuzzy_app_name",
            "scenario": "scenario",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "appNames": False,
            "appStatuses": False,
            "fuzzyAppName": False,
            "scenario": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class GeneratedPagination(Model):
    """Generated pagination (generated.Pagination)

    Properties:
        first: (first) OPTIONAL str

        last: (last) OPTIONAL str

        next_: (next) OPTIONAL str

        previous: (previous) OPTIONAL str
    """

    # region fields

    first: str  # OPTIONAL
    last: str  # OPTIONAL
    next_: str  # OPTIONAL
    previous: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_first(self, value: str) -> GeneratedPagination:
        self.first = value
        return self

    def with_last(self, value: str) -> GeneratedPagination:
        self.last = value
        return self

    def with_next(self, value: str) -> GeneratedPagination:
        self.next_ = value
        return self

    def with_previous(self, value: str) -> GeneratedPagination:
        self.previous = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "first"):
            result["first"] = str(self.first)
        elif include_empty:
            result["first"] = ""
        if hasattr(self, "last"):
            result["last"] = str(self.last)
        elif include_empty:
            result["last"] = ""
        if hasattr(self, "next_"):
            result["next"] = str(self.next_)
        elif include_empty:
            result["next"] = ""
        if hasattr(self, "previous"):
            result["previous"] = str(self.previous)
        elif include_empty:
            result["previous"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        first: Optional[str] = None,
        last: Optional[str] = None,
        next_: Optional[str] = None,
        previous: Optional[str] = None,
        **kwargs,
    ) -> GeneratedPagination:
        instance = cls()
        if first is not None:
            instance.first = first
        if last is not None:
            instance.last = last
        if next_ is not None:
            instance.next_ = next_
        if previous is not None:
            instance.previous = previous
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> GeneratedPagination:
        instance = cls()
        if not dict_:
            return instance
        if "first" in dict_ and dict_["first"] is not None:
            instance.first = str(dict_["first"])
        elif include_empty:
            instance.first = ""
        if "last" in dict_ and dict_["last"] is not None:
            instance.last = str(dict_["last"])
        elif include_empty:
            instance.last = ""
        if "next" in dict_ and dict_["next"] is not None:
            instance.next_ = str(dict_["next"])
        elif include_empty:
            instance.next_ = ""
        if "previous" in dict_ and dict_["previous"] is not None:
            instance.previous = str(dict_["previous"])
        elif include_empty:
            instance.previous = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, GeneratedPagination]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[GeneratedPagination]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        GeneratedPagination, List[GeneratedPagination], Dict[Any, GeneratedPagination]
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "first": "first",
            "last": "last",
            "next": "next_",
            "previous": "previous",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "first": False,
            "last": False,
            "next": False,
            "previous": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model

from ..models.apimodel_autoscaling_response import ApimodelAutoscalingResponse
from ..models.apimodel_cpu_response import ApimodelCPUResponse
from ..models.apimodel_memory_response import ApimodelMemoryResponse
from ..models.apimodel_replica_response import ApimodelReplicaResponse


class ApimodelAppItem(Model):
    """Apimodel app item (apimodel.AppItem)

    Properties:
        app_id: (appId) REQUIRED str

        app_name: (appName) REQUIRED str

        app_status: (appStatus) REQUIRED str

        created_at: (createdAt) REQUIRED str

        game_name: (gameName) REQUIRED str

        scenario: (scenario) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        app_repo_arn: (appRepoArn) OPTIONAL str

        app_repo_url: (appRepoUrl) OPTIONAL str

        autoscaling: (autoscaling) OPTIONAL ApimodelAutoscalingResponse

        base_path: (basePath) OPTIONAL str

        cpu: (cpu) OPTIONAL ApimodelCPUResponse

        deleted_at: (deletedAt) OPTIONAL str

        deployment_created_at: (deploymentCreatedAt) OPTIONAL str

        deployment_id: (deploymentId) OPTIONAL str

        deployment_image_tag: (deploymentImageTag) OPTIONAL str

        description: (description) OPTIONAL str

        memory: (memory) OPTIONAL ApimodelMemoryResponse

        message: (message) OPTIONAL str

        replica: (replica) OPTIONAL ApimodelReplicaResponse

        service_public_url: (servicePublicURL) OPTIONAL str

        service_url: (serviceURL) OPTIONAL str
    """

    # region fields

    app_id: str  # REQUIRED
    app_name: str  # REQUIRED
    app_status: str  # REQUIRED
    created_at: str  # REQUIRED
    game_name: str  # REQUIRED
    scenario: str  # REQUIRED
    updated_at: str  # REQUIRED
    app_repo_arn: str  # OPTIONAL
    app_repo_url: str  # OPTIONAL
    autoscaling: ApimodelAutoscalingResponse  # OPTIONAL
    base_path: str  # OPTIONAL
    cpu: ApimodelCPUResponse  # OPTIONAL
    deleted_at: str  # OPTIONAL
    deployment_created_at: str  # OPTIONAL
    deployment_id: str  # OPTIONAL
    deployment_image_tag: str  # OPTIONAL
    description: str  # OPTIONAL
    memory: ApimodelMemoryResponse  # OPTIONAL
    message: str  # OPTIONAL
    replica: ApimodelReplicaResponse  # OPTIONAL
    service_public_url: str  # OPTIONAL
    service_url: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_app_id(self, value: str) -> ApimodelAppItem:
        self.app_id = value
        return self

    def with_app_name(self, value: str) -> ApimodelAppItem:
        self.app_name = value
        return self

    def with_app_status(self, value: str) -> ApimodelAppItem:
        self.app_status = value
        return self

    def with_created_at(self, value: str) -> ApimodelAppItem:
        self.created_at = value
        return self

    def with_game_name(self, value: str) -> ApimodelAppItem:
        self.game_name = value
        return self

    def with_scenario(self, value: str) -> ApimodelAppItem:
        self.scenario = value
        return self

    def with_updated_at(self, value: str) -> ApimodelAppItem:
        self.updated_at = value
        return self

    def with_app_repo_arn(self, value: str) -> ApimodelAppItem:
        self.app_repo_arn = value
        return self

    def with_app_repo_url(self, value: str) -> ApimodelAppItem:
        self.app_repo_url = value
        return self

    def with_autoscaling(self, value: ApimodelAutoscalingResponse) -> ApimodelAppItem:
        self.autoscaling = value
        return self

    def with_base_path(self, value: str) -> ApimodelAppItem:
        self.base_path = value
        return self

    def with_cpu(self, value: ApimodelCPUResponse) -> ApimodelAppItem:
        self.cpu = value
        return self

    def with_deleted_at(self, value: str) -> ApimodelAppItem:
        self.deleted_at = value
        return self

    def with_deployment_created_at(self, value: str) -> ApimodelAppItem:
        self.deployment_created_at = value
        return self

    def with_deployment_id(self, value: str) -> ApimodelAppItem:
        self.deployment_id = value
        return self

    def with_deployment_image_tag(self, value: str) -> ApimodelAppItem:
        self.deployment_image_tag = value
        return self

    def with_description(self, value: str) -> ApimodelAppItem:
        self.description = value
        return self

    def with_memory(self, value: ApimodelMemoryResponse) -> ApimodelAppItem:
        self.memory = value
        return self

    def with_message(self, value: str) -> ApimodelAppItem:
        self.message = value
        return self

    def with_replica(self, value: ApimodelReplicaResponse) -> ApimodelAppItem:
        self.replica = value
        return self

    def with_service_public_url(self, value: str) -> ApimodelAppItem:
        self.service_public_url = value
        return self

    def with_service_url(self, value: str) -> ApimodelAppItem:
        self.service_url = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "app_id"):
            result["appId"] = str(self.app_id)
        elif include_empty:
            result["appId"] = ""
        if hasattr(self, "app_name"):
            result["appName"] = str(self.app_name)
        elif include_empty:
            result["appName"] = ""
        if hasattr(self, "app_status"):
            result["appStatus"] = str(self.app_status)
        elif include_empty:
            result["appStatus"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "game_name"):
            result["gameName"] = str(self.game_name)
        elif include_empty:
            result["gameName"] = ""
        if hasattr(self, "scenario"):
            result["scenario"] = str(self.scenario)
        elif include_empty:
            result["scenario"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "app_repo_arn"):
            result["appRepoArn"] = str(self.app_repo_arn)
        elif include_empty:
            result["appRepoArn"] = ""
        if hasattr(self, "app_repo_url"):
            result["appRepoUrl"] = str(self.app_repo_url)
        elif include_empty:
            result["appRepoUrl"] = ""
        if hasattr(self, "autoscaling"):
            result["autoscaling"] = self.autoscaling.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["autoscaling"] = ApimodelAutoscalingResponse()
        if hasattr(self, "base_path"):
            result["basePath"] = str(self.base_path)
        elif include_empty:
            result["basePath"] = ""
        if hasattr(self, "cpu"):
            result["cpu"] = self.cpu.to_dict(include_empty=include_empty)
        elif include_empty:
            result["cpu"] = ApimodelCPUResponse()
        if hasattr(self, "deleted_at"):
            result["deletedAt"] = str(self.deleted_at)
        elif include_empty:
            result["deletedAt"] = ""
        if hasattr(self, "deployment_created_at"):
            result["deploymentCreatedAt"] = str(self.deployment_created_at)
        elif include_empty:
            result["deploymentCreatedAt"] = ""
        if hasattr(self, "deployment_id"):
            result["deploymentId"] = str(self.deployment_id)
        elif include_empty:
            result["deploymentId"] = ""
        if hasattr(self, "deployment_image_tag"):
            result["deploymentImageTag"] = str(self.deployment_image_tag)
        elif include_empty:
            result["deploymentImageTag"] = ""
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        if hasattr(self, "memory"):
            result["memory"] = self.memory.to_dict(include_empty=include_empty)
        elif include_empty:
            result["memory"] = ApimodelMemoryResponse()
        if hasattr(self, "message"):
            result["message"] = str(self.message)
        elif include_empty:
            result["message"] = ""
        if hasattr(self, "replica"):
            result["replica"] = self.replica.to_dict(include_empty=include_empty)
        elif include_empty:
            result["replica"] = ApimodelReplicaResponse()
        if hasattr(self, "service_public_url"):
            result["servicePublicURL"] = str(self.service_public_url)
        elif include_empty:
            result["servicePublicURL"] = ""
        if hasattr(self, "service_url"):
            result["serviceURL"] = str(self.service_url)
        elif include_empty:
            result["serviceURL"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        app_id: str,
        app_name: str,
        app_status: str,
        created_at: str,
        game_name: str,
        scenario: str,
        updated_at: str,
        app_repo_arn: Optional[str] = None,
        app_repo_url: Optional[str] = None,
        autoscaling: Optional[ApimodelAutoscalingResponse] = None,
        base_path: Optional[str] = None,
        cpu: Optional[ApimodelCPUResponse] = None,
        deleted_at: Optional[str] = None,
        deployment_created_at: Optional[str] = None,
        deployment_id: Optional[str] = None,
        deployment_image_tag: Optional[str] = None,
        description: Optional[str] = None,
        memory: Optional[ApimodelMemoryResponse] = None,
        message: Optional[str] = None,
        replica: Optional[ApimodelReplicaResponse] = None,
        service_public_url: Optional[str] = None,
        service_url: Optional[str] = None,
        **kwargs,
    ) -> ApimodelAppItem:
        instance = cls()
        instance.app_id = app_id
        instance.app_name = app_name
        instance.app_status = app_status
        instance.created_at = created_at
        instance.game_name = game_name
        instance.scenario = scenario
        instance.updated_at = updated_at
        if app_repo_arn is not None:
            instance.app_repo_arn = app_repo_arn
        if app_repo_url is not None:
            instance.app_repo_url = app_repo_url
        if autoscaling is not None:
            instance.autoscaling = autoscaling
        if base_path is not None:
            instance.base_path = base_path
        if cpu is not None:
            instance.cpu = cpu
        if deleted_at is not None:
            instance.deleted_at = deleted_at
        if deployment_created_at is not None:
            instance.deployment_created_at = deployment_created_at
        if deployment_id is not None:
            instance.deployment_id = deployment_id
        if deployment_image_tag is not None:
            instance.deployment_image_tag = deployment_image_tag
        if description is not None:
            instance.description = description
        if memory is not None:
            instance.memory = memory
        if message is not None:
            instance.message = message
        if replica is not None:
            instance.replica = replica
        if service_public_url is not None:
            instance.service_public_url = service_public_url
        if service_url is not None:
            instance.service_url = service_url
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelAppItem:
        instance = cls()
        if not dict_:
            return instance
        if "appId" in dict_ and dict_["appId"] is not None:
            instance.app_id = str(dict_["appId"])
        elif include_empty:
            instance.app_id = ""
        if "appName" in dict_ and dict_["appName"] is not None:
            instance.app_name = str(dict_["appName"])
        elif include_empty:
            instance.app_name = ""
        if "appStatus" in dict_ and dict_["appStatus"] is not None:
            instance.app_status = str(dict_["appStatus"])
        elif include_empty:
            instance.app_status = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "gameName" in dict_ and dict_["gameName"] is not None:
            instance.game_name = str(dict_["gameName"])
        elif include_empty:
            instance.game_name = ""
        if "scenario" in dict_ and dict_["scenario"] is not None:
            instance.scenario = str(dict_["scenario"])
        elif include_empty:
            instance.scenario = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "appRepoArn" in dict_ and dict_["appRepoArn"] is not None:
            instance.app_repo_arn = str(dict_["appRepoArn"])
        elif include_empty:
            instance.app_repo_arn = ""
        if "appRepoUrl" in dict_ and dict_["appRepoUrl"] is not None:
            instance.app_repo_url = str(dict_["appRepoUrl"])
        elif include_empty:
            instance.app_repo_url = ""
        if "autoscaling" in dict_ and dict_["autoscaling"] is not None:
            instance.autoscaling = ApimodelAutoscalingResponse.create_from_dict(
                dict_["autoscaling"], include_empty=include_empty
            )
        elif include_empty:
            instance.autoscaling = ApimodelAutoscalingResponse()
        if "basePath" in dict_ and dict_["basePath"] is not None:
            instance.base_path = str(dict_["basePath"])
        elif include_empty:
            instance.base_path = ""
        if "cpu" in dict_ and dict_["cpu"] is not None:
            instance.cpu = ApimodelCPUResponse.create_from_dict(
                dict_["cpu"], include_empty=include_empty
            )
        elif include_empty:
            instance.cpu = ApimodelCPUResponse()
        if "deletedAt" in dict_ and dict_["deletedAt"] is not None:
            instance.deleted_at = str(dict_["deletedAt"])
        elif include_empty:
            instance.deleted_at = ""
        if "deploymentCreatedAt" in dict_ and dict_["deploymentCreatedAt"] is not None:
            instance.deployment_created_at = str(dict_["deploymentCreatedAt"])
        elif include_empty:
            instance.deployment_created_at = ""
        if "deploymentId" in dict_ and dict_["deploymentId"] is not None:
            instance.deployment_id = str(dict_["deploymentId"])
        elif include_empty:
            instance.deployment_id = ""
        if "deploymentImageTag" in dict_ and dict_["deploymentImageTag"] is not None:
            instance.deployment_image_tag = str(dict_["deploymentImageTag"])
        elif include_empty:
            instance.deployment_image_tag = ""
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        if "memory" in dict_ and dict_["memory"] is not None:
            instance.memory = ApimodelMemoryResponse.create_from_dict(
                dict_["memory"], include_empty=include_empty
            )
        elif include_empty:
            instance.memory = ApimodelMemoryResponse()
        if "message" in dict_ and dict_["message"] is not None:
            instance.message = str(dict_["message"])
        elif include_empty:
            instance.message = ""
        if "replica" in dict_ and dict_["replica"] is not None:
            instance.replica = ApimodelReplicaResponse.create_from_dict(
                dict_["replica"], include_empty=include_empty
            )
        elif include_empty:
            instance.replica = ApimodelReplicaResponse()
        if "servicePublicURL" in dict_ and dict_["servicePublicURL"] is not None:
            instance.service_public_url = str(dict_["servicePublicURL"])
        elif include_empty:
            instance.service_public_url = ""
        if "serviceURL" in dict_ and dict_["serviceURL"] is not None:
            instance.service_url = str(dict_["serviceURL"])
        elif include_empty:
            instance.service_url = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelAppItem]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelAppItem]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[ApimodelAppItem, List[ApimodelAppItem], Dict[Any, ApimodelAppItem]]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "appId": "app_id",
            "appName": "app_name",
            "appStatus": "app_status",
            "createdAt": "created_at",
            "gameName": "game_name",
            "scenario": "scenario",
            "updatedAt": "updated_at",
            "appRepoArn": "app_repo_arn",
            "appRepoUrl": "app_repo_url",
            "autoscaling": "autoscaling",
            "basePath": "base_path",
            "cpu": "cpu",
            "deletedAt": "deleted_at",
            "deploymentCreatedAt": "deployment_created_at",
            "deploymentId": "deployment_id",
            "deploymentImageTag": "deployment_image_tag",
            "description": "description",
            "memory": "memory",
            "message": "message",
            "replica": "replica",
            "servicePublicURL": "service_public_url",
            "serviceURL": "service_url",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "appId": True,
            "appName": True,
            "appStatus": True,
            "createdAt": True,
            "gameName": True,
            "scenario": True,
            "updatedAt": True,
            "appRepoArn": False,
            "appRepoUrl": False,
            "autoscaling": False,
            "basePath": False,
            "cpu": False,
            "deletedAt": False,
            "deploymentCreatedAt": False,
            "deploymentId": False,
            "deploymentImageTag": False,
            "description": False,
            "memory": False,
            "message": False,
            "replica": False,
            "servicePublicURL": False,
            "serviceURL": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model-init.j2

"""Auto-generated package that contains models used by the Custom Service Manager."""

__version__ = "1.20.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .apimodel_app_item import ApimodelAppItem
from .apimodel_autoscaling_request import ApimodelAutoscalingRequest
from .apimodel_autoscaling_response import ApimodelAutoscalingResponse
from .apimodel_bulk_subscribe_item import ApimodelBulkSubscribeItem
from .apimodel_bulk_subscribe_request import ApimodelBulkSubscribeRequest
from .apimodel_cpu_request import ApimodelCPURequest
from .apimodel_cpu_response import ApimodelCPUResponse
from .apimodel_create_app_v2_request import ApimodelCreateAppV2Request
from .apimodel_create_deployment_v2_request import ApimodelCreateDeploymentV2Request
from .apimodel_create_deployment_v2_response import ApimodelCreateDeploymentV2Response
from .apimodel_csm_app_limits_response import ApimodelCSMAppLimitsResponse
from .apimodel_get_app_image_list_v2_data_item import ApimodelGetAppImageListV2DataItem
from .apimodel_get_app_image_list_v2_data_item_image_scan_result import (
    ApimodelGetAppImageListV2DataItemImageScanResult,
)
from .apimodel_get_app_image_list_v2_data_item_image_scan_status import (
    ApimodelGetAppImageListV2DataItemImageScanStatus,
)
from .apimodel_get_app_image_list_v2_response import ApimodelGetAppImageListV2Response
from .apimodel_get_app_list_v2_request import ApimodelGetAppListV2Request
from .apimodel_get_app_list_v2_response import ApimodelGetAppListV2Response
from .apimodel_get_deployment_list_v2_data_item import (
    ApimodelGetDeploymentListV2DataItem,
)
from .apimodel_get_deployment_list_v2_request import ApimodelGetDeploymentListV2Request
from .apimodel_get_deployment_list_v2_response import (
    ApimodelGetDeploymentListV2Response,
)
from .apimodel_get_list_of_configurations_v2_data_item import (
    ApimodelGetListOfConfigurationsV2DataItem,
)
from .apimodel_get_list_of_configurations_v2_response import (
    ApimodelGetListOfConfigurationsV2Response,
)
from .apimodel_get_notification_subscriber_item import (
    ApimodelGetNotificationSubscriberItem,
)
from .apimodel_get_notification_subscriber_list_response import (
    ApimodelGetNotificationSubscriberListResponse,
)
from .apimodel_get_notification_subscriber_status_response import (
    ApimodelGetNotificationSubscriberStatusResponse,
)
from .apimodel_increase_limit_form_request import ApimodelIncreaseLimitFormRequest
from .apimodel_memory_request import ApimodelMemoryRequest
from .apimodel_memory_response import ApimodelMemoryResponse
from .apimodel_replica_request import ApimodelReplicaRequest
from .apimodel_replica_response import ApimodelReplicaResponse
from .apimodel_save_configuration_v2_request import ApimodelSaveConfigurationV2Request
from .apimodel_save_configuration_v2_response import ApimodelSaveConfigurationV2Response
from .apimodel_self_subscribe_notification_request import (
    ApimodelSelfSubscribeNotificationRequest,
)
from .apimodel_subscribe_notification_request import (
    ApimodelSubscribeNotificationRequest,
)
from .apimodel_subscribe_notification_response import (
    ApimodelSubscribeNotificationResponse,
)
from .apimodel_subscriber_item_request import ApimodelSubscriberItemRequest
from .apimodel_subscriber_item_response import ApimodelSubscriberItemResponse
from .apimodel_update_app_resource_request import ApimodelUpdateAppResourceRequest
from .apimodel_update_app_v2_request import ApimodelUpdateAppV2Request
from .apimodel_update_configuration_v2_request import (
    ApimodelUpdateConfigurationV2Request,
)
from .apimodel_update_configuration_v2_response import (
    ApimodelUpdateConfigurationV2Response,
)
from .generated_create_app_v1_request import GeneratedCreateAppV1Request
from .generated_create_app_v1_response import GeneratedCreateAppV1Response
from .generated_create_deployment_v1_request import GeneratedCreateDeploymentV1Request
from .generated_create_deployment_v1_response import GeneratedCreateDeploymentV1Response
from .generated_delete_app_images_v1_request import GeneratedDeleteAppImagesV1Request
from .generated_get_app_image_list_v1_data_item import (
    GeneratedGetAppImageListV1DataItem,
)
from .generated_get_app_image_list_v1_data_item_image_scan_status import (
    GeneratedGetAppImageListV1DataItemImageScanStatus,
)
from .generated_get_app_image_list_v1_response import GeneratedGetAppImageListV1Response
from .generated_get_app_list_v1_data_item import GeneratedGetAppListV1DataItem
from .generated_get_app_list_v1_request import GeneratedGetAppListV1Request
from .generated_get_app_list_v1_response import GeneratedGetAppListV1Response
from .generated_get_app_release_v1_response import GeneratedGetAppReleaseV1Response
from .generated_get_app_v1_response import GeneratedGetAppV1Response
from .generated_get_deployment_list_v1_data_item import (
    GeneratedGetDeploymentListV1DataItem,
)
from .generated_get_deployment_list_v1_request import (
    GeneratedGetDeploymentListV1Request,
)
from .generated_get_deployment_list_v1_response import (
    GeneratedGetDeploymentListV1Response,
)
from .generated_get_deployment_v1_response import GeneratedGetDeploymentV1Response
from .generated_get_list_of_configurations_v1_data_item import (
    GeneratedGetListOfConfigurationsV1DataItem,
)
from .generated_get_list_of_configurations_v1_response import (
    GeneratedGetListOfConfigurationsV1Response,
)
from .generated_pagination import GeneratedPagination
from .generated_save_configuration_v1_request import GeneratedSaveConfigurationV1Request
from .generated_save_configuration_v1_response import (
    GeneratedSaveConfigurationV1Response,
)
from .generated_start_app_v1_request import GeneratedStartAppV1Request
from .generated_start_app_v1_response import GeneratedStartAppV1Response
from .generated_stop_app_v1_request import GeneratedStopAppV1Request
from .generated_stop_app_v1_response import GeneratedStopAppV1Response
from .generated_update_app_v1_request import GeneratedUpdateAppV1Request
from .generated_update_app_v1_response import GeneratedUpdateAppV1Response
from .generated_update_configuration_v1_request import (
    GeneratedUpdateConfigurationV1Request,
)
from .generated_update_configuration_v1_response import (
    GeneratedUpdateConfigurationV1Response,
)
from .log_app_message_declaration import LogAppMessageDeclaration
from .model_csm_autoscaling_defaults import ModelCSMAutoscalingDefaults
from .model_image_scan_finding import ModelImageScanFinding
from .response_error_response import ResponseErrorResponse



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: wrapper.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import get_namespace as get_services_namespace
from accelbyte_py_sdk.core import run_request
from accelbyte_py_sdk.core import run_request_async
from accelbyte_py_sdk.core import deprecated
from accelbyte_py_sdk.core import same_doc_as

from ..models import GeneratedCreateAppV1Request
from ..models import GeneratedCreateAppV1Response
from ..models import GeneratedGetAppListV1Request
from ..models import GeneratedGetAppListV1Response
from ..models import GeneratedGetAppReleaseV1Response
from ..models import GeneratedGetAppV1Response
from ..models import GeneratedStartAppV1Request
from ..models import GeneratedStartAppV1Response
from ..models import GeneratedStopAppV1Request
from ..models import GeneratedStopAppV1Response
from ..models import GeneratedUpdateAppV1Request
from ..models import GeneratedUpdateAppV1Response
from ..models import ResponseErrorResponse

from ..operations.app import CreateAppV1
from ..operations.app import DeleteAppV1
from ..operations.app import GetAppListV1
from ..operations.app import GetAppReleaseV1
from ..operations.app import GetAppV1
from ..operations.app import StartAppV1
from ..operations.app import StopAppV1
from ..operations.app import UpdateAppV1


@deprecated
@same_doc_as(CreateAppV1)
def create_app_v1(
    app: str,
    body: GeneratedCreateAppV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Creates new App for AB-Extend Customers (CreateAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [CREATE]`

    Creates new App for AB-Extend Customers

    Available scenario:
    - scenario 1: `function-override`
    - scenario 2: `service-extension`
    - scenario 3: `event-handler`

    Default: `function-override`

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}

        method: PUT

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedCreateAppV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedCreateAppV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateAppV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(CreateAppV1)
async def create_app_v1_async(
    app: str,
    body: GeneratedCreateAppV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Creates new App for AB-Extend Customers (CreateAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [CREATE]`

    Creates new App for AB-Extend Customers

    Available scenario:
    - scenario 1: `function-override`
    - scenario 2: `service-extension`
    - scenario 3: `event-handler`

    Default: `function-override`

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}

        method: PUT

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedCreateAppV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedCreateAppV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateAppV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(DeleteAppV1)
def delete_app_v1(
    app: str,
    forced: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete App by Åpp Name (DeleteAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [DELETE]`

    Delete App by given DeploymentID

    This endpoint intended to delete ECR repo, ECR manifests, service images, uninstall helm-release,
    and update the deleted_at in DB by given App Name.

    Required: Valid Access Token

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}

        method: DELETE

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        forced: (forced) OPTIONAL str in query

    Responses:
        204: No Content -

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteAppV1.create(
        app=app,
        forced=forced,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(DeleteAppV1)
async def delete_app_v1_async(
    app: str,
    forced: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete App by Åpp Name (DeleteAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [DELETE]`

    Delete App by given DeploymentID

    This endpoint intended to delete ECR repo, ECR manifests, service images, uninstall helm-release,
    and update the deleted_at in DB by given App Name.

    Required: Valid Access Token

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}

        method: DELETE

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        forced: (forced) OPTIONAL str in query

    Responses:
        204: No Content -

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteAppV1.create(
        app=app,
        forced=forced,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetAppListV1)
def get_app_list_v1(
    body: GeneratedGetAppListV1Request,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Gets the List of Apps for AB-Extend Customer (GetAppListV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [READ]`

    Gets the List of Apps for AB-Extend Customer

    Available scenario:
    - scenario 3: `event-handler`
    - scenario 1: `function-override`
    - scenario 2: `service-extension`

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps

        method: POST

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedGetAppListV1Request in body

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - GeneratedGetAppListV1Response

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAppListV1.create(
        body=body,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetAppListV1)
async def get_app_list_v1_async(
    body: GeneratedGetAppListV1Request,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Gets the List of Apps for AB-Extend Customer (GetAppListV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [READ]`

    Gets the List of Apps for AB-Extend Customer

    Available scenario:
    - scenario 3: `event-handler`
    - scenario 1: `function-override`
    - scenario 2: `service-extension`

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps

        method: POST

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedGetAppListV1Request in body

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - GeneratedGetAppListV1Response

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAppListV1.create(
        body=body,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetAppReleaseV1)
def get_app_release_v1(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Gets the Latest Release Version info of this App (GetAppReleaseV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [READ]`

    Gets the Latest Release Version info of this App

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/release

        method: GET

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedGetAppReleaseV1Response

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAppReleaseV1.create(
        app=app,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetAppReleaseV1)
async def get_app_release_v1_async(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Gets the Latest Release Version info of this App (GetAppReleaseV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [READ]`

    Gets the Latest Release Version info of this App

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/release

        method: GET

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedGetAppReleaseV1Response

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAppReleaseV1.create(
        app=app,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(GetAppV1)
def get_app_v1(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Gets the App By Name (GetAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [READ]`

    Gets the App By Name

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}

        method: GET

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedGetAppV1Response

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAppV1.create(
        app=app,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(GetAppV1)
async def get_app_v1_async(
    app: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Gets the App By Name (GetAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [READ]`

    Gets the App By Name

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}

        method: GET

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedGetAppV1Response

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAppV1.create(
        app=app,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(StartAppV1)
def start_app_v1(
    app: str,
    body: GeneratedStartAppV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Starts the Application (StartAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [UPDATE]`

    Starts the Application

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/start

        method: PUT

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedStartAppV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedStartAppV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = StartAppV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(StartAppV1)
async def start_app_v1_async(
    app: str,
    body: GeneratedStartAppV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Starts the Application (StartAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [UPDATE]`

    Starts the Application

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/start

        method: PUT

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedStartAppV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedStartAppV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = StartAppV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(StopAppV1)
def stop_app_v1(
    app: str,
    body: GeneratedStopAppV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Stops the Application (StopAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [UPDATE]`

    Stops the Application

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/stop

        method: PUT

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedStopAppV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedStopAppV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = StopAppV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(StopAppV1)
async def stop_app_v1_async(
    app: str,
    body: GeneratedStopAppV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Stops the Application (StopAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [UPDATE]`

    Stops the Application

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}/stop

        method: PUT

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedStopAppV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedStopAppV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = StopAppV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(UpdateAppV1)
def update_app_v1(
    app: str,
    body: GeneratedUpdateAppV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update App Partially (UpdateAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [UPDATE]`

    Update App Partially

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}

        method: PATCH

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedUpdateAppV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedUpdateAppV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateAppV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(UpdateAppV1)
async def update_app_v1_async(
    app: str,
    body: GeneratedUpdateAppV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update App Partially (UpdateAppV1)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:APP [UPDATE]`

    Update App Partially

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/apps/{app}

        method: PATCH

        tags: ["App"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedUpdateAppV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - GeneratedUpdateAppV1Response (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = UpdateAppV1.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: wrapper.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import get_namespace as get_services_namespace
from accelbyte_py_sdk.core import run_request
from accelbyte_py_sdk.core import run_request_async
from accelbyte_py_sdk.core import same_doc_as

from ..models import ApimodelCreateDeploymentV2Request
from ..models import ApimodelCreateDeploymentV2Response
from ..models import ApimodelGetDeploymentListV2DataItem
from ..models import ApimodelGetDeploymentListV2Request
from ..models import ApimodelGetDeploymentListV2Response
from ..models import ResponseErrorResponse

from ..operations.deployment_v2 import CreateDeploymentV2
from ..operations.deployment_v2 import DeleteDeploymentV2
from ..operations.deployment_v2 import GetDeploymentV2
from ..operations.deployment_v2 import GetListOfDeploymentV2


@same_doc_as(CreateDeploymentV2)
def create_deployment_v2(
    app: str,
    body: ApimodelCreateDeploymentV2Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Creates Deployment (CreateDeploymentV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:DEPLOYMENT [CREATE]`

    Creates Deployment for extend service

    This Endpoint will create new deployment and apply all of the secrets and variable as environment variable

    Required:
    - Valid Access Token
    - Valid Image Version Created on Uploading image with docker or other method
    - Valid App name
    - Valid Namespace name

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/deployments

        method: POST

        tags: ["Deployment V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelCreateDeploymentV2Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApimodelCreateDeploymentV2Response

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateDeploymentV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateDeploymentV2)
async def create_deployment_v2_async(
    app: str,
    body: ApimodelCreateDeploymentV2Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Creates Deployment (CreateDeploymentV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:DEPLOYMENT [CREATE]`

    Creates Deployment for extend service

    This Endpoint will create new deployment and apply all of the secrets and variable as environment variable

    Required:
    - Valid Access Token
    - Valid Image Version Created on Uploading image with docker or other method
    - Valid App name
    - Valid Namespace name

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/deployments

        method: POST

        tags: ["Deployment V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelCreateDeploymentV2Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApimodelCreateDeploymentV2Response

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = CreateDeploymentV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteDeploymentV2)
def delete_deployment_v2(
    deployment_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Deployment by Deployment ID (DeleteDeploymentV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:DEPLOYMENT [DELETE]`

    Delete Deployment information by given DeploymentID

    This endpoint will delete release object for the service deployment in the cluster

    Required: Valid Access Token

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/deployments/{deploymentId}

        method: DELETE

        tags: ["Deployment V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment_id: (deploymentId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteDeploymentV2.create(
        deployment_id=deployment_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteDeploymentV2)
async def delete_deployment_v2_async(
    deployment_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete Deployment by Deployment ID (DeleteDeploymentV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:DEPLOYMENT [DELETE]`

    Delete Deployment information by given DeploymentID

    This endpoint will delete release object for the service deployment in the cluster

    Required: Valid Access Token

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/deployments/{deploymentId}

        method: DELETE

        tags: ["Deployment V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment_id: (deploymentId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteDeploymentV2.create(
        deployment_id=deployment_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetDeploymentV2)
def get_deployment_v2(
    deployment_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Deployment by Deployment ID (GetDeploymentV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:DEPLOYMENT [READ]`

    Get Deployment information by given DeploymentID

    This endpoint intended to get Deployment information.

    Required: Valid Access Token

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/deployments/{deploymentId}

        method: GET

        tags: ["Deployment V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment_id: (deploymentId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelGetDeploymentListV2DataItem

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetDeploymentV2.create(
        deployment_id=deployment_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetDeploymentV2)
async def get_deployment_v2_async(
    deployment_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get Deployment by Deployment ID (GetDeploymentV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:DEPLOYMENT [READ]`

    Get Deployment information by given DeploymentID

    This endpoint intended to get Deployment information.

    Required: Valid Access Token

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/deployments/{deploymentId}

        method: GET

        tags: ["Deployment V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        deployment_id: (deploymentId) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApimodelGetDeploymentListV2DataItem

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetDeploymentV2.create(
        deployment_id=deployment_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetListOfDeploymentV2)
def get_list_of_deployment_v2(
    body: ApimodelGetDeploymentListV2Request,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fetches the List of Deployments (GetListOfDeploymentV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:DEPLOYMENT [READ]`

    Filters the List of Deployments

    This endpoint intended to get list of Deployment done for the given publisher and game.
    Request :
    - appIds : list of app ids
    - deploymentIds: list of deployment ids
    - statuses:
    deployment-in-progress
    deployment-failed
    deployment-timeout
    deployment-running
    deployment-down
    Required: Valid Access Token

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/deployments

        method: POST

        tags: ["Deployment V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelGetDeploymentListV2Request in body

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ApimodelGetDeploymentListV2Response

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetListOfDeploymentV2.create(
        body=body,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetListOfDeploymentV2)
async def get_list_of_deployment_v2_async(
    body: ApimodelGetDeploymentListV2Request,
    limit: Optional[int] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Fetches the List of Deployments (GetListOfDeploymentV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:DEPLOYMENT [READ]`

    Filters the List of Deployments

    This endpoint intended to get list of Deployment done for the given publisher and game.
    Request :
    - appIds : list of app ids
    - deploymentIds: list of deployment ids
    - statuses:
    deployment-in-progress
    deployment-failed
    deployment-timeout
    deployment-running
    deployment-down
    Required: Valid Access Token

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/deployments

        method: POST

        tags: ["Deployment V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelGetDeploymentListV2Request in body

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ApimodelGetDeploymentListV2Response

        400: Bad Request - ResponseErrorResponse (Bad Request)

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetListOfDeploymentV2.create(
        body=body,
        limit=limit,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: wrapper.j2

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import HeaderStr
from accelbyte_py_sdk.core import get_namespace as get_services_namespace
from accelbyte_py_sdk.core import run_request
from accelbyte_py_sdk.core import run_request_async
from accelbyte_py_sdk.core import same_doc_as

from ..models import ApimodelGetAppImageListV2Response
from ..models import GeneratedDeleteAppImagesV1Request
from ..models import ResponseErrorResponse

from ..operations.image_v2 import DeleteAppImagesV2
from ..operations.image_v2 import GetAppImageListV2


@same_doc_as(DeleteAppImagesV2)
def delete_app_images_v2(
    app: str,
    body: GeneratedDeleteAppImagesV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete app images (DeleteAppImagesV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:IMAGE [DELETE]`

    Deletes list of provided image tags from the app image repository

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/images

        method: DELETE

        tags: ["Image V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedDeleteAppImagesV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteAppImagesV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteAppImagesV2)
async def delete_app_images_v2_async(
    app: str,
    body: GeneratedDeleteAppImagesV1Request,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete app images (DeleteAppImagesV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:IMAGE [DELETE]`

    Deletes list of provided image tags from the app image repository

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/images

        method: DELETE

        tags: ["Image V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED GeneratedDeleteAppImagesV1Request in body

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content -

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = DeleteAppImagesV2.create(
        app=app,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetAppImageListV2)
def get_app_image_list_v2(
    app: str,
    cached: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a list of container images (GetAppImageListV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:IMAGE [READ]`

    Get a list of container images

    Default 'cached' parameter is 'true'

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/images

        method: GET

        tags: ["Image V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        cached: (cached) OPTIONAL str in query

    Responses:
        200: OK - ApimodelGetAppImageListV2Response

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAppImageListV2.create(
        app=app,
        cached=cached,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetAppImageListV2)
async def get_app_image_list_v2_async(
    app: str,
    cached: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a list of container images (GetAppImageListV2)

    Required permission : `ADMIN:NAMESPACE:{namespace}:EXTEND:IMAGE [READ]`

    Get a list of container images

    Default 'cached' parameter is 'true'

    Properties:
        url: /csm/v2/admin/namespaces/{namespace}/apps/{app}/images

        method: GET

        tags: ["Image V2"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app: (app) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        cached: (cached) OPTIONAL str in query

    Responses:
        200: OK - ApimodelGetAppImageListV2Response

        401: Unauthorized - ResponseErrorResponse (Unauthorized)

        403: Forbidden - ResponseErrorResponse (Forbidden)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace()
        if error:
            return None, error
    request = GetAppImageListV2.create(
        app=app,
        cached=cached,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model

from ..models.apimodel_get_app_image_list_v2_data_item_image_scan_result import (
    ApimodelGetAppImageListV2DataItemImageScanResult,
)
from ..models.apimodel_get_app_image_list_v2_data_item_image_scan_status import (
    ApimodelGetAppImageListV2DataItemImageScanStatus,
)


class ApimodelGetAppImageListV2DataItem(Model):
    """Apimodel get app image list V2 data item (apimodel.GetAppImageListV2DataItem)

    Properties:
        image_digest: (imageDigest) REQUIRED str

        image_tag: (imageTag) REQUIRED str

        size: (size) REQUIRED int

        image_scan_result: (imageScanResult) OPTIONAL ApimodelGetAppImageListV2DataItemImageScanResult

        image_scan_status: (imageScanStatus) OPTIONAL ApimodelGetAppImageListV2DataItemImageScanStatus

        updated_at: (updatedAt) OPTIONAL str
    """

    # region fields

    image_digest: str  # REQUIRED
    image_tag: str  # REQUIRED
    size: int  # REQUIRED
    image_scan_result: ApimodelGetAppImageListV2DataItemImageScanResult  # OPTIONAL
    image_scan_status: ApimodelGetAppImageListV2DataItemImageScanStatus  # OPTIONAL
    updated_at: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_image_digest(self, value: str) -> ApimodelGetAppImageListV2DataItem:
        self.image_digest = value
        return self

    def with_image_tag(self, value: str) -> ApimodelGetAppImageListV2DataItem:
        self.image_tag = value
        return self

    def with_size(self, value: int) -> ApimodelGetAppImageListV2DataItem:
        self.size = value
        return self

    def with_image_scan_result(
        self, value: ApimodelGetAppImageListV2DataItemImageScanResult
    ) -> ApimodelGetAppImageListV2DataItem:
        self.image_scan_result = value
        return self

    def with_image_scan_status(
        self, value: ApimodelGetAppImageListV2DataItemImageScanStatus
    ) -> ApimodelGetAppImageListV2DataItem:
        self.image_scan_status = value
        return self

    def with_updated_at(self, value: str) -> ApimodelGetAppImageListV2DataItem:
        self.updated_at = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "image_digest"):
            result["imageDigest"] = str(self.image_digest)
        elif include_empty:
            result["imageDigest"] = ""
        if hasattr(self, "image_tag"):
            result["imageTag"] = str(self.image_tag)
        elif include_empty:
            result["imageTag"] = ""
        if hasattr(self, "size"):
            result["size"] = int(self.size)
        elif include_empty:
            result["size"] = 0
        if hasattr(self, "image_scan_result"):
            result["imageScanResult"] = self.image_scan_result.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result[
                "imageScanResult"
            ] = ApimodelGetAppImageListV2DataItemImageScanResult()
        if hasattr(self, "image_scan_status"):
            result["imageScanStatus"] = self.image_scan_status.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result[
                "imageScanStatus"
            ] = ApimodelGetAppImageListV2DataItemImageScanStatus()
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        image_digest: str,
        image_tag: str,
        size: int,
        image_scan_result: Optional[
            ApimodelGetAppImageListV2DataItemImageScanResult
        ] = None,
        image_scan_status: Optional[
            ApimodelGetAppImageListV2DataItemImageScanStatus
        ] = None,
        updated_at: Optional[str] = None,
        **kwargs,
    ) -> ApimodelGetAppImageListV2DataItem:
        instance = cls()
        instance.image_digest = image_digest
        instance.image_tag = image_tag
        instance.size = size
        if image_scan_result is not None:
            instance.image_scan_result = image_scan_result
        if image_scan_status is not None:
            instance.image_scan_status = image_scan_status
        if updated_at is not None:
            instance.updated_at = updated_at
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelGetAppImageListV2DataItem:
        instance = cls()
        if not dict_:
            return instance
        if "imageDigest" in dict_ and dict_["imageDigest"] is not None:
            instance.image_digest = str(dict_["imageDigest"])
        elif include_empty:
            instance.image_digest = ""
        if "imageTag" in dict_ and dict_["imageTag"] is not None:
            instance.image_tag = str(dict_["imageTag"])
        elif include_empty:
            instance.image_tag = ""
        if "size" in dict_ and dict_["size"] is not None:
            instance.size = int(dict_["size"])
        elif include_empty:
            instance.size = 0
        if "imageScanResult" in dict_ and dict_["imageScanResult"] is not None:
            instance.image_scan_result = (
                ApimodelGetAppImageListV2DataItemImageScanResult.create_from_dict(
                    dict_["imageScanResult"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.image_scan_result = (
                ApimodelGetAppImageListV2DataItemImageScanResult()
            )
        if "imageScanStatus" in dict_ and dict_["imageScanStatus"] is not None:
            instance.image_scan_status = (
                ApimodelGetAppImageListV2DataItemImageScanStatus.create_from_dict(
                    dict_["imageScanStatus"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.image_scan_status = (
                ApimodelGetAppImageListV2DataItemImageScanStatus()
            )
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelGetAppImageListV2DataItem]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelGetAppImageListV2DataItem]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelGetAppImageListV2DataItem,
        List[ApimodelGetAppImageListV2DataItem],
        Dict[Any, ApimodelGetAppImageListV2DataItem],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "imageDigest": "image_digest",
            "imageTag": "image_tag",
            "size": "size",
            "imageScanResult": "image_scan_result",
            "imageScanStatus": "image_scan_status",
            "updatedAt": "updated_at",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "imageDigest": True,
            "imageTag": True,
            "size": True,
            "imageScanResult": False,
            "imageScanStatus": False,
            "updatedAt": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: operation-init.j2

"""Auto-generated package that contains models used by the Custom Service Manager."""

__version__ = "1.20.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .bulk_save_subscription__e5ef74 import BulkSaveSubscriptionAppNotificationV2
from .delete_subscription_app_0003d9 import DeleteSubscriptionAppNotificationByUserIDV2
from .delete_subscription_app_84e329 import DeleteSubscriptionAppNotificationV2
from .get_notification_subscr_4bced3 import GetNotificationSubscriberListV2
from .get_subscription_v2_handler import GetSubscriptionV2Handler
from .subscribe_app_notification_v2 import SubscribeAppNotificationV2
from .subscribe_v2_handler import SubscribeV2Handler
from .unsubscribe_v2_handler import UnsubscribeV2Handler



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: model.j2

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from accelbyte_py_sdk.core import Model


class ApimodelSaveConfigurationV2Request(Model):
    """Apimodel save configuration V2 request (apimodel.SaveConfigurationV2Request)

    Properties:
        config_name: (configName) REQUIRED str

        source: (source) REQUIRED str

        value: (value) REQUIRED str

        apply_mask: (applyMask) OPTIONAL bool

        description: (description) OPTIONAL str
    """

    # region fields

    config_name: str  # REQUIRED
    source: str  # REQUIRED
    value: str  # REQUIRED
    apply_mask: bool  # OPTIONAL
    description: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_config_name(self, value: str) -> ApimodelSaveConfigurationV2Request:
        self.config_name = value
        return self

    def with_source(self, value: str) -> ApimodelSaveConfigurationV2Request:
        self.source = value
        return self

    def with_value(self, value: str) -> ApimodelSaveConfigurationV2Request:
        self.value = value
        return self

    def with_apply_mask(self, value: bool) -> ApimodelSaveConfigurationV2Request:
        self.apply_mask = value
        return self

    def with_description(self, value: str) -> ApimodelSaveConfigurationV2Request:
        self.description = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "config_name"):
            result["configName"] = str(self.config_name)
        elif include_empty:
            result["configName"] = ""
        if hasattr(self, "source"):
            result["source"] = str(self.source)
        elif include_empty:
            result["source"] = ""
        if hasattr(self, "value"):
            result["value"] = str(self.value)
        elif include_empty:
            result["value"] = ""
        if hasattr(self, "apply_mask"):
            result["applyMask"] = bool(self.apply_mask)
        elif include_empty:
            result["applyMask"] = False
        if hasattr(self, "description"):
            result["description"] = str(self.description)
        elif include_empty:
            result["description"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        config_name: str,
        source: str,
        value: str,
        apply_mask: Optional[bool] = None,
        description: Optional[str] = None,
        **kwargs,
    ) -> ApimodelSaveConfigurationV2Request:
        instance = cls()
        instance.config_name = config_name
        instance.source = source
        instance.value = value
        if apply_mask is not None:
            instance.apply_mask = apply_mask
        if description is not None:
            instance.description = description
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelSaveConfigurationV2Request:
        instance = cls()
        if not dict_:
            return instance
        if "configName" in dict_ and dict_["configName"] is not None:
            instance.config_name = str(dict_["configName"])
        elif include_empty:
            instance.config_name = ""
        if "source" in dict_ and dict_["source"] is not None:
            instance.source = str(dict_["source"])
        elif include_empty:
            instance.source = ""
        if "value" in dict_ and dict_["value"] is not None:
            instance.value = str(dict_["value"])
        elif include_empty:
            instance.value = ""
        if "applyMask" in dict_ and dict_["applyMask"] is not None:
            instance.apply_mask = bool(dict_["applyMask"])
        elif include_empty:
            instance.apply_mask = False
        if "description" in dict_ and dict_["description"] is not None:
            instance.description = str(dict_["description"])
        elif include_empty:
            instance.description = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelSaveConfigurationV2Request]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelSaveConfigurationV2Request]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelSaveConfigurationV2Request,
        List[ApimodelSaveConfigurationV2Request],
        Dict[Any, ApimodelSaveConfigurationV2Request],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "configName": "config_name",
            "source": "source",
            "value": "value",
            "applyMask": "apply_mask",
            "description": "description",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "configName": True,
            "source": True,
            "value": True,
            "applyMask": False,
            "description": False,
        }

    # endregion static methods
